{-------------------------------------------------------------}
{
{    Interface simplificada para Secure Sockets Layer
{
{    Restrio: apenas 1 conexo por vez
{
{    Autor: Jose' Antonio Borges
{
{    Em 15/02/2008
{
{-------------------------------------------------------------}


unit dvssl;

interface

uses
    dvcrt,
    sysutils,
    windows,
    winSock;

function inicializa_ctx_SSL: boolean;
function conectaSSL (sock: integer; ender: string): boolean;
procedure fechaSSL;
function recebeSSL (buf: PChar; len: integer): integer;
function enviaSSL (buf: PChar; len: integer): integer;
function temDadoSsl: boolean;
function receiveBuf (sock: integer; buf: PChar; len: integer; flags: integer): integer;
function sendBuf (sock: integer; buf: pchar; len: integer; flags: integer): integer;

const
    portaSSLPOP3  = 995;
    portaSSLSMTP  = 465;
    portaSSLHTTPS = 443;
    portaSSLFTPS  = 990;

var
    sslSock: integer;                           { soquete origem da conexo SSL }

implementation
const
  SSL_ERROR_NONE = 0;
  SSL_ERROR_SSL = 1;
  SSL_ERROR_WANT_READ = 2;
  SSL_ERROR_WANT_WRITE = 3;
  SSL_ERROR_WANT_X509_LOOKUP = 4;
  SSL_ERROR_SYSCALL = 5; //look at error stack/return value/errno
  SSL_ERROR_ZERO_RETURN = 6;

var
    SSLActive: boolean;                         { bibliotecas SSL j ativadas }
    SSLConnected: boolean;

{-------------------------------------------------------------}

procedure criacontextossl; cdecl; external 'encapsulassl.dll';
procedure cInicializaSSL; cdecl; external 'encapsulassl.dll' name 'inicializassl';
function cconectassl(sock: integer): integer; cdecl; external 'encapsulassl.dll' name 'conectassl';
procedure criasocketssl(ender: pChar); cdecl; external 'encapsulassl.dll';
function obtemerrossl: integer; cdecl; external 'encapsulassl.dll';
function tentaconectar: integer; cdecl; external 'encapsulassl.dll';
function lessl(buf: pChar; tam: integer): integer; cdecl; external 'encapsulassl.dll';
function escrevessl(buf: pChar; tam: integer): integer; cdecl; external 'encapsulassl.dll';
function ctemdadossl: integer; cdecl; external 'encapsulassl.dll' name 'temdadossl';
procedure desligassl; cdecl; external 'encapsulassl.dll';

function inicializa_ctx_SSL: boolean;
begin
    inicializa_ctx_SSL := true;

    if not SSLActive then
        begin
            SSLActive := true;
            cInicializaSSL;
            criaContextoSSL;
        end;
end;

{-------------------------------------------------------------}

function conectaSSL (sock: integer; ender: string): boolean;
var conOk: boolean;
begin
criaSocketSSL(@ender[1]);
    cConectaSSL(sock);
    conOk := tentaconectar > 0;
    conectaSSL := conOk;
    if conOk then sslSock := sock;
    SSLConnected := conOk;
end;

{-------------------------------------------------------------}

procedure fechaSSL;
begin
    desligaSSL;
    sslSock := 0;
    SSLConnected := false;
end;

{-------------------------------------------------------------}

function recebeSSL (buf: PChar; len: integer): integer;
var
    readBlocked: boolean;
    r, e: integer;

begin
    repeat
        readBlocked := false;
        r :=  leSSL (buf, len);

        e := obtemErroSSL;
        case e of

            SSL_ERROR_NONE:
                begin
                    buf[r] := #$0;
                    break;
                end;

            SSL_ERROR_SYSCALL,      (* Retornado pelo gmail *)
            SSL_ERROR_ZERO_RETURN:  (* End of data *)
                begin
                    r := -1;
                    break;
                end;

            SSL_ERROR_WANT_READ:    readBlocked := true;
            SSL_ERROR_WANT_WRITE:   ;

        else
            writeln ('Erro SSL: ', e);
            r := -1;
            break;
        end;

        delay (100);

    until not readBlocked;

    recebeSSL := r;
end;

{-------------------------------------------------------------}

function enviaSSL (buf: PChar; len: integer): integer;
var
    writeBlocked: boolean;
    r, e: integer;

begin
    repeat
        writeBlocked := false;

        r := escreveSSL(buf, len);
        e := obtemErroSSL;
        case e of

            SSL_ERROR_NONE:
                    break;

            SSL_ERROR_SYSCALL,      (* Retornado pelo gmail *)
            SSL_ERROR_ZERO_RETURN:  (* End of data *)
                begin
                    r := -1;
                    break;
                end;

            SSL_ERROR_WANT_READ:    ;
            SSL_ERROR_WANT_WRITE:   writeBlocked := true;

        else
            r := -1;
            break;
        end;

        delay (100);

    until not writeBlocked;

    enviaSSL := r;
end;

{-------------------------------------------------------------}

function temDadoSsl: boolean;
begin
    if SSLConnected then
        temDadoSSL := cTemDadoSSL <> 0
    else
        temDadoSSL := false;
end;

{-------------------------------------------------------------}

function receiveBuf (sock: integer; buf: PChar; len: integer; flags: integer): integer;
begin
    if SSLConnected and (sock = sslSock) then
        receiveBuf := RecebeSSL (buf, len)
    else
        receiveBuf := winSock.recv (sock, buf^, len, flags);
end;

{-------------------------------------------------------------}

function sendBuf (sock: integer; buf: pchar; len: integer; flags: integer): integer;
begin
    if SSLConnected and (sock = sslSock) then
        sendBuf := enviaSSL (buf, len)
    else
        sendBuf := winsock.send (sock, buf^, len, flags);
end;

begin
    SSLActive := false;
    SSLConnected := false;
    sslSock := 0;
end.
